	.z80
subttl CP/M Cold and Warm Boot Module
	page
;
;	Teletek Enterprises, Inc.
;
;================================================================
;	  SYSTEMASTER CP/M 2.2 COLD AND WARM BOOT MODULE
;
;		      revision number:  C.00
;		     revision date:  06-11-82
;
;	This module initializes the bios on cold boot and
;	boots in the CCP and BDOS of CP/M 2.2 on a warm 
;	boot.
;
;	The CP/M system is contained on the system diskette 
;	in the following format. Both the eight inch and the
;	five and a quarter diskette is formatted with first
;	track as single density 128 byte sectors and second
;	and remainder tracks as double density 256 byte 
;	sectors.
;
;
;		Eight inch diskette.
;
;				      TRACK	SECTORS	
;	COLD START LOADER	-	0	  1-2
;	CCP and BDOS		-	0	  3-26
;				    and 1	  1-10
;	BIOS			-	1	 11-25
;	DISKETTE VOLUME ID SEC.	-	1	    26
;
;		Five and a Quarter inch diskette.
;
;				      TRACK	SECTORS	
;	COLD START LOADER	-	0	  1-2
;	CCP and BDOS		-	0	  3-18
;				    and 1	  1-14
;	BIOS			-	1	 15-18
;				    and 1	  1-11
;	(unused)		-	1	 12-17
;	DISKETTE VOLUME ID SEC.	-	1	    18
;
;	DISKETTE VOLUME ID sector is reserved for future release
;	of the system. 	It will be used to optimize the thru-put
;	of double density disk I/O.
;
;	This module requires the following equates to be set
;
;	In SMBIOS.MAC - DSKDAT
;
;	In SMVCTR.MOD - SMVCTR
;
;	In SMDVAR.MOD - HSTWRT, HSTDSK, HSTACT, UNACNT, TRACK
;			and DMAADR
;
;	In SMPIOS.MOD - ANDSER, JRX, and SLIST
;
;================================================================
;
;	local equates
;
cnfg	defl	4		;configure flag
cld	defl	4		;current logical drive no.
drvsztb defl	44h		;drive size table st. add.
step8	defl	48h		;specify and cnbyte addr csl.
bauda	defl	4dh		;baud rate for sio a csl.
baudb	defl	4fh		;baud rate for sio b csl.
sp8dat	defl	dskdat+22	;specify and cnbyte addr loc.
;
;
boot:	;entry from cold start loader
	call	signon
boot0:	ld	(cld),a		;clear current drive
;
wboot:	;warm boot (reboot) cp/m
	ld	a,(hstwrt)	;host buffer need to
	or	a		;written?
	jr	z,wbt0
	ld	a,(hstdsk)
	add	a,'A'
	ld	(fod),a
	call	pstr
	db	cr,lf,'BDOS Err on '
fod:	dc	'A: File Open'
	call	conin
	cp	3
	call	nz,writehst
wbt0:	ld	sp,80h		;set up stack
	ld	a,(drvsztb)
	call	stsiz
	xor	a
	call	stmode		;set up s.den mode
	xor	a
	call	sthdrv		;boot from drive 0
wbt1	cal	home		;recalibrate
	jr	nz,wbt1
	ld	a,3
	call	stsect		;set sector 3
	xor	a
	ld	(track),a
wbt2:	call	seeker		;seek track 0
	jr	nz,wbt2
	ld	hl,ccp
	ld	a,(btns1)	;load no of sectors
	ld	e,a
	call	read		;read in first track of data
	jr	nz,wbter
;
	ld	a,40h		;set to d.density
	call	stmode		;and boot in remainder of bdos
	ld	a,1		;on second track
	call	stsect
	ld	(track),a
wbt3:	call	seeker
	jr	nz,wbt3
	ld	hl,ccp
	ld	de,(btns2)	;load nsc1 and nsc2
	ld	a,d		;get nsec1
	rrca			;divide by 2
	add	a,h		;update ccp address
	ld	h,a		;assumed to be on even page bound.
	call	read		;read last group of sectors
	jr	z,wbt6
;
wbter:	;warm boot error
	ld	a,2
	call	error		;call error routine
	jr	wbt1
;
wbt6:	ld	a,0c3h		;set up cp/m jump vectors
	ld	(0),a
	ld	(5),a
	ld	hl,bios+3
	ld	(1),hl
	ld	hl,bdos
	ld	(6),hl
	ld	hl,80h		;set init dma addr
	ld	(dmaadr),hl
	xor	a		;init host flags
	ld	(hstact),a
	ld	(unacnt),a
	ld	a,(cld)		;get logical drive
	ld	c,a
	jp	ccp
;
btns2:	ds	1
btns1:	ds	1
;
;================================================================
;
;this code is only used just after the cold start loader.
;it initializes several parameters and prints the sign on
;message.  It disables itself after first entry to the routine.
;
signon:	call	init		;initialize board (SMINIT.MOD)
	ld	a,0c9h
	ld	(signon),a	;put a ret to disable
	ld	hl,smvctr	;get vector entry address
	ld	a,0c3h
	ld	(40H),a
	ld	(41h),hl	;store it for utilities
	ld	hl,step8	;transfer specify words and
	ld	de,sp8dat	;CNBYTE to the bios
	ld	bc,5
	ldir
	ld	a,(drvsztb)
	or	a		;check size of drive a
	ld	hl,24*256+10	;8"?
	jr	nz,sn1		;yes
	ld	hl,16*256+14	;no, 5.25"
sn1:	ld	(btns2),hl
	ld	a,(cnfg)	;check configuration
	bit	7,a		;any protocol?
	jr	z,sn2		;no
	ld	hl,0		;yes, enable protocol
	ld	(slist),hl
	bit	6,a		;etx/ack protocol?
	jr	z,sn2		;no, x-on/x-off
	ld	(jrx),hl
sn2:	bit	5,a		;mask off output byte?
	jr	z,sn3		;yes
	ld	(andser),hl	;no
sn3:	ld	bc,2*256+ctc0	;set up list baud rate
	ld	hl,bauda
	otir
;
	ld	a,(cnfg)	;check configuration
	bit	4,a		;auto boot mode ?
	jr	z,sn4		;no
	xor	a		;yes
	ld	(bctr),a	;disable baud search (SMPIOS.MOD)
	ld	bc,2*256+ctc1	;assign console baud rate
	ld	hl,baudb
	otir
	jr	sn5
;
sn4:;		not auto boot mode, so sio b baud not preset.
;		wait until proper baud rate on sio b is found
;		(interrupt ser input will search until found)
;		See SMPIOS.MOD
;
	call	conin
;
;		next send signon message
;
sn5:	call	pstr		;print sign on message (SMPSIO.MOD)
	db	cr,lf,lf,lf,lf,lf
	db	'Teletek Enterprises, Inc.',cr,lf
	db	'SYSTEMASTER '
	db	memsz/10+'0',memsz mod 10 +'0'
	db	'k CP/M vers 2.2',cr,lf
	db	'[Release C.00]',cr,lf+80h
	xor	a
	ld	(hstwrt),a	;clear flag
	ret			;on exit a must = 0
;
;	end of module
;
