	.z80
	subttl CP/M Disk I/O Module
	page
;
;	Teletek Enterprises, Inc.
;
;================================================================
;		 SYSTEMASTER CP/M DISK I/O MODULE
;
;		      revision number:  B.00
;		     revision date:  05-25-82
;
;	This module contains all of the code for interfacing
;	CP/M 2.2 disk I/O to the SYSTEMASTER.    Companion
;	module SMDVAR.MOD contains the CP/M disk variables 
;	and constants.
;
;	Entries to this module.
;
;		HOME   - Home the drive (disabled)
;		SELDSK - Select the drive
;		SETTRK - Set the track number
;		SETSEC - Set the sector number
;		SETDMA - Set the DMA address
;		SECTRAN- Translate the sector number
;		GETSEC - Get the CP/M RECORD (READ)
;		PUTSEC - Put the CP/M RECORD (WRITE)
;
;	The module expects the following equates to be available
;	during assembly.
;
;		STMODE - Set the 765 mode for I/O
;		STSIZE - Set size of drive
;		STHDRV - Set the head and drive
;		STSECT - Set the sector
;		SPEC   - Do a specify
;		RECAL  - Recalibrate the drive
;		SEEK   - Set the track and seek
;		READ   - Read sector(s)
;		WRITE  - Write sector(s)
;		READID - Read ID of first available sector
;		RETSAD - Return status address location
;		RETDST - Return drive status
;		RETRTC - Return address of Clock/calender
;
;================================================================
;	cp/m disk equates
;
drvmsk	defl	43h		;drive mask
drvsztb	defl	44h		;drive size table
step8	defl	48h		;step rate for 8"
step5	defl	4Ah		;step rate for 5.25"
;
;set total number of logical drives:
ndrvs	defl	4
;
;set appropriate number of tracks:
ntrks8	defl	77		;8"
ntrks5	defl	35		;5.25"
;
;	cp/m to host disk constants
blksiz	defl	2048		;cp/m allocation size (d.dens)
hstsiz	defl	256		;host disk sector size
hstblk	defl	hstsiz/128	;cp/m sects/host buff
secmsk	defl	hstblk-1	;sector mask
secshf	defl	1		;log2(hstblk)
;
;
;	bdos constants on entry to write
wrall	defl	0		;write to allocated
wrdir	defl	1		;write to directory
wrual	defl	2		;write to unallocated
;
;
;	# of bytes for drive parameters
ndpar	defl	5		;1 size
				;2 density
				;3 dph (low)
				;4 dph (high)
				;5 cp/m sec/trk
;
cld	defl	4		;current logical drive
;
;================================================================
;	start of code
;
home:	;move head to track 0
	ret			;disabled, done by seldsk
;
seldsk:	;select a drive
	ld	a,c
	cp	ndrvs		;drive > # of drives?
	jp	nc,selerr	;yes, then err
	ld	(sekdsk),a
	ld	b,0
	ld	hl,drprtb	;get drive parameter table
	rept	ndpar		;and point to drive entry
	add	hl,bc
	endm
	bit	0,e		;already logged in?
	jp	nz,sdpar	;yes
	push	hl		;no, find density
	ex	de,hl
	ld	hl,drvsztb
	add	hl,bc		;point to drive size
	push	af
	ld	a,(hl)
	ld	(de),a		;save size
	call	setsiz		;set drive size
	pop	af
	call	setdrv		;set drive
	call	homer		;recalibrate
	jp	nz,sdnl		;if err, drive not on line
	inc	a
	ld	(track),a
	ld	a,3		;try to find density
sd0:	ld	(rwtrys),a	;at least 3 times
	call	seeker		;seek track 1
	ld	a,40h		;set density
	call	stmode
	call	readid		;read 1st id
	ld	a,40h
	jr	z,sd1		;jp if double density
	sub	a
	call	stmode
	call	readid		;single density?
	jr	z,sd1		;yes
	ld	a,(rwtrys)
	dec	a
	jr	nz,sd0
	jp	sderr
;
sd1:	pop	hl
	push	hl
	inc	hl
	ld	(hl),a		;save density
	push	af
	push	hl
	ld	a,(sekdsk)
	rlca
	rlca
	rlca
	rlca
	ld	e,a
	ld	d,0
	ld	hl,dpbase
	add	hl,de
	ex	de,hl
	pop	hl
	inc	hl
	ld	(hl),e		;store disk parameter header
	inc	hl
	ld	(hl),d
	ex	de,hl
	pop	af		;get density
	or	a		;single?
	push	de
	jr	nz,sd2		;no, double
	ld	de,dpb8s
	ld	bc,xlt8
	ld	a,(drsz)	;get current drive size
	or	a		;8" drive?
	jr	nz,sd5		;yes
	ld	de,dpb5s	;no, 5.25"
	ld	bc,xlt5
	jr	sd5
sd2:	call	recal		;double dens, check if
	ld	a,(sekdsk)	;drive is double sided
	or	4		;set head 1
	push	hl
	call	setdrv
	pop	hl
	call	readid		;single sided drive?
	ld	b,a		;nz= yes, z= no
	ld	a,(drsz)
	or	a		;8" drive?
	ld	a,b
	jr	nz,sd3		;yes
	or	a		;double sided?
	ld	de,dpb5d
	jr	nz,sd4		;no, single sided
	ld	de,dpb5dd	;yes
	jr	sd4
sd3:	or	a		;double sided?
	ld	de,dpb8d
	jr	nz,sd4		;yes
	ld	de,dpb8dd	;no, double sided
sd4:	ld	bc,0
sd5:	ld	(hl),c		;store sector trans table
	inc	hl
	ld	(hl),b
	ld	bc,9
	add	hl,bc
	ld	(hl),e		;store disk parameter block
	inc	hl
	ld	(hl),d
	ld	a,(de)		;get cp/m sec/trk
	pop	hl
	inc	hl
	ld	(hl),a
	pop	hl
;
sdpar:	;set seek disk parameters
	ld	a,(hl)		;size
	ld	(sekdsz),a
	inc	hl
	ld	a,(hl)		;density
	ld	(sekden),a
	inc	hl
	ld	e,(hl)		;disk parameter header
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	a,(hl)
	ld	(sekcst),a	;cp/m sec/trk
	ex	de,hl
	ret
;
sderr:	;density error
	ld	a,3
	call	error
sdnl:	;drive not on line
	pop	hl
selerr:	;select error
	xor	a
	ld	(cld),a		;reset logical drive to a
	ld	hl,0
	ret
;
settrk:	;set track
	ld	(sektrk),bc
	ret
;
setsec:	;set sector
	ld	a,c
	ld	(seksec),a
	ret
;
setdma:	;set dma address
	ld	(dmaadr),bc
	ret
;
sectran: ;translate the sector given by bc
	ld	h,b
	ld	l,c
	ld	a,d
	or	e
	ret	z		;return if de=0
	ex	de,hl
	add	hl,bc
	ld	l,(hl)
	ld	h,0
	ret
;
getsec:		;read the selected cp/m sector
	xor	a
	ld	(unacnt),a
	ld	a,(sekden)	;check density
	or	a
	jp	z,reads		;jp if single
	ld	(readop),a	;read operation
	ld	(rsflag),a	;must read data
	ld	a,wrual
	ld	(wrtype),a	;treat as unalloc
	jr	rwoper		;to perform the read
;
putsec:		;write the selected cp/m sector
	ld	a,(sekden)	;check density
	or	a
	jp	z,writs		;jp if single density
	xor	a
	ld	(readop),a	;not a read operation
	ld	a,c		;write type in c
	ld	(wrtype),a
	cp	wrual		;write unallocated?
	jr	nz,chkuna	;check for unalloc
	;write to unallocated, set parameters
	ld	a,blksiz/128	;next unalloc recs
	ld	(unacnt),a
	ld	hl,sekdsk
	ld	de,unadsk
	ld	bc,4
	ldir
;
chkuna:	;check for write to unallocated sector
	ld	a,(unacnt)	;any unalloc remain?
	or	a
	jr	z,alloc		;skip if not
	;more unallocated records remain
	dec	a		;unacnt = unacnt-1
	ld	(unacnt),a
	ld	hl,unadsk
	ld	b,4
	call	ckbkeq
	jr	nz,alloc	;no match go allocate	
;
;	match, move to next sector for future ref
	dec	hl		;point to unasec
	inc	(hl)		;unasec = unasec+1
	ld	a,(sekcst)	;get cpm sec/trk
	ld	b,a
	ld	a,(hl)		;end of track?
	cp	b
	jr	c,noovf		;skip if no overflow
;
;	overflow to next track
	ld	(hl),0		;unasec = 0
	ld	hl,(unatrk)
	inc	hl		;unatrk = unatrk+1
	ld	(unatrk),hl
;
noovf:	;match found, mark as unnecessary read
	xor	a		;0 to accumulator
	ld	(rsflag),a	;rsflag = 0
	jr	rwoper		;to perform the write
;
alloc:	;not an unallocated record, requires pre-read
	xor	a
	ld	(unacnt),a	;unacnt = 0
	inc	a
	ld	(rsflag),a	;rsflag = 1
;
;	common code for read and write follows
rwoper:	;enter here to perform the read/write
	xor	a
	ld	(erflag),a	;no errors (yet)
	ld	a,(seksec)	;compute host sector
	rra			;shift right
	ld	(sekhst),a	;host sector to seek
;
;	active host sector?
	ld	hl,hstact	;host active flag
	ld	a,(hl)
	ld	(hl),1		;always becomes 1
	or	a		;was it already?
	jr	z,filhst	;fill host if not
;
;	check for host - seek match
	ld	hl,hstdsk
	ld	b,3
	call	ckbkeq
	jr	nz,nomatch
;
;	same disk, same track, same buffer?
	ld	a,(sekhst)
	ld	hl,hstsec	;sekhst=hstsec?
	cp	(hl)
	jr	z,match		;skip if match
;
;	proper disk, but not correct sector
nomatch: ld	a,(hstwrt)	;host written?
	or	a
	call	nz,writehst	;clear host buff
;
;	move seek parameters to host
filhst:	ld	hl,sekpar
	ld	de,hstpar
	ld	bc,npar
	ldir
	ld	a,(sekhst)
	ld	(hstsec),a
	ld	a,(rsflag)	;need to read?
	or	a
	call	nz,readhst	;yes, if 1
	xor	a
	ld	(hstwrt),a	;no pending write
;
;	copy data to or from buffer
match:	ld	hl,hstbuf
	ld	bc,128
	ld	a,(seksec)
	and	secmsk
	jr	z,mtch0
	add	hl,bc
mtch0:	ld	de,(dmaadr)	;get/put cp/m data
	ld	a,(readop)	;which way?
	or	a
	jr	nz,rwmove	;skip if read
;
;	write operation, mark and switch direction
	ld	a,1
	ld	(hstwrt),a	;hstwrt = 1
	ex	de,hl		;source/dest swap
;
;	bc initially 128, hl is source, de is dest
rwmove:	ldir
;
;	data has been moved to/from host buffer
	ld	a,(wrtype)	;write type
	cp	wrdir		;to directory?
	ld	a,(erflag)	;in case of errors
	ret	nz		;no further processing
;
;	clear host buffer for directory write
	or	a		;errors?
	ret	nz		;skip if so
	call	writehst
	ld	a,(erflag)
	ret	
;
;	checks block located at address in hl reg
;	against block located at sekdsk
;	returns	z flag set if equal,clear if not equal	
ckbkeq:	ld	de,sekdsk
ckb1:	ld	a,(de)
	cp	(hl)
	ret	nz
	inc	hl
	inc	de
	djnz	ckb1
	ret
;
;	physical write to host disk.
writehst:
	;hstdsk = host disk #, hsttrk = host track #,
	;hstsec = host sect #. write "hstsiz" bytes
	;from hstbuf and return error flag in erflag.
	;return erflag non-zero if error
	xor	a
	ld	(hstwrt),a	;clear flag
	jr	rwhst
;
;	physical read from host disk.
readhst:
	;hstdsk = host disk #, hsttrk = host track #,
	;hstsec = host sect #. read "hstsiz" bytes
	;into hstbuf and return error flag in erflag.
	ld	a,1
;
;	fdc dependent routines
rwhst:	ld	(rdop),a
	ld	a,(hstdsz)	;set the drive size
	call	setsiz
	ld	a,40h
	call	stmode		;set double density
	ld	hl,(xlt)	;translate the sector
	ld	a,(hstsec)
	ld	e,a
	ld	d,0
	add	hl,de
	ld	a,(hl)
	call	stsect
	ld	a,(ntrks)
	ld	b,a
	ld	a,(hsttrk)	;setup track byte
	ld	(track),a
	ld	c,0
	sub	b		;subtract # of tracks
	jr	c,rwh1
	ld	c,4		;set head 1 on overflow
	ld	(track),a
rwh1:	ld	a,(hstdsk)
	or	c
	ld	hl,hstbuf
;
;	read or write on floppy disk
rwfdc:	ld	(actdma),hl	;set memory pointer
	call	setdrv		;set head/drive
rwf1:	ld	a,10		;try at least 10 times
rwf2:	ld	(rwtrys),a
	and	3
	call	z,readid	;to set internal track reg
	call	seeker
	call	z,actrw
	ld	(erflag),a
	ret	z
	ld	a,(rwtrys)
	dec	a
	jr	nz,rwf2
	ld	a,(rdop)
	inc	a
	call	error
	ret	z
	call	homer
	jr	rwf1
;
actrw:	;actual read or write
	ld	hl,(actdma)	;get actual dma addr
	ld	e,1
	ld	a,(rdop)
	or	a		;read operation?
	jp	nz,read		;yes
	jp	write		;no, must be write
;
;
writs:	;write the selected cp/m sector - single density
	xor	a
	jr	rws
;
reads:	;read the selected cp/m sector - single density
	ld	a,1
rws:	ld	(rdop),a
	ld	a,(sekdsz)	;set the drive size
	call	setsiz
	xor	a
	call	stmode		;set single density
	ld	a,(seksec)
	call	stsect		;set the sector
	ld	a,(sektrk)
	ld	(track),a	;set the track
	ld	a,(sekdsk)
	ld	hl,(dmaadr)
	jr	rwfdc
;
setsiz:	;set drive size
	ld	b,a
	ld	a,(drsz)	;check current size
	cp	b		;the same?
	ret	z		;yes, do nothing
	ld	a,b
stsiz:	ld	(drsz),a
	or	a
	ld	hl,xlt8
	ld	a,ntrks8
	jr	nz,stsz
	ld	hl,xlt5
	ld	a,ntrks5
stsz:
	ld	(xlt),hl
	ld	(ntrks),a
	ld	a,(drsz)
	jp	stsize
;
setdrv:	;set the drive and head
	ld	hl,drvmsk
	and	(hl)		;mask drive
	jp	sthdrv
;
homer:	;actual recalibrate routine
	ld	a,3
hm0:	ld	(hmtrys),a
	call	recal
	ret	z
	ld	a,(hmtrys)
	dec	a
	jr	nz,hm0
	call	error
	jr	nz,homer
	or	a
	ret
;
seeker:	;actual seek routine
	ld	a,(track)
	call	seek
	ret	z
	sub	a
	call	error
	jr	nz,seeker
	or	a
	ret
;
;error printing routine.  enter with a= error as follows:
;     error	type
;	0	seek (or recalibrate)
;	1	write
;	2	read
;	3	not on line
;	4	density
error:	push	af
	call	retsad
	ld	a,(hl)
	and	3
	add	a,'A'
	ld	(erd),a
	call	pstr
	db	bell,cr,lf,'Disk Err on '
erd:	dc	'A: '
	pop	af
	or	a
	jr	nz,err0
	call	pstr
	dc	'Seek'
	jr	err4
err0:	dec	a
	jr	nz,err1
	call	pstr
	dc	'Write'
	jr	err4
err1:	dec	a
	jr	nz,err2
	call	pstr
	dc	'Read'
	jr	err4
err2:	call	pstr
	dc	'Density'
	ret
err4:	call	pstr
	dc	','
	bit	7,(hl)		;abnormal termination?
	jr	z,err5
	call	pstr
	dc	' AT'
	jr	err13
err5:	bit	4,(hl)		;equipment check?
	jr	z,err6
	call	pstr
	dc	' EC'
	jr	err13
err6:	bit	3,(hl)		;not ready?
	jr	z,err7
	call	pstr
	dc	' NR'
	jr	err13
err7:	inc	hl		;point to st1
	bit	5,(hl)		;data error?
	jr	z,err8
	call	pstr
	dc	' DE'
err8:	bit	2,(hl)		;no data?
	jr	z,err9
	call	pstr
	dc	' ND'
err9:	bit	1,(hl)		;write protect?
	jr	z,err10
	call	pstr
	dc	' NW'
err10:	bit	0,(hl)		;missing addr mark?
	jr	z,err11
	call	pstr
	dc	' MA'
err11:	inc	hl		;point to st2
	bit	4,(hl)		;wrong cylinder?
	jr	z,err12
	call	pstr
	dc	' WC'
err12:	call	pstr
	dc	' (T='
	call	pdec		;print cylinder (track)
	call	pstr
	dc	' H='
	call	pdec		;print head
	call	pstr
	dc	' S='
	call	pdec		;print sector
	ld	a,')'
	call	outchr
err13:	call	conin		;wait for any key
	cp	cr		;return error?
	ret	z
	sub	3		;^C (reboot)?
	jp	z,boot0
	call	pstr
	db	cr,lf+80h
	ret
;
pdec:	ld	de,10		;print (hl+1) in decimal
	inc	hl
	ld	a,(hl)
pd0:	inc	d
	sub	e
	jr	nc,pd0
	ld	e,a
	ld	a,d
	dec	a		;don't print leading zero
	jr	z,pd1
	add	a,'0'
	call	outchr
pd1:	ld	a,e
	add	a,'0'+10
	jp	outchr

;
;	end of module
;
