	.z80
	subttl	DISK/RTC control module
	page
;
;	Teletek Enterprises, Inc.
;
;================================================================
;		SYSTEMASTER DISK AND RTC CONTROL MODULE
;
;			 revision number:  B.00
;			revision date:  05-25-82
;
;	This module controls the disk I/O and the Real-time-clock
;	of the SysteMaster. It has 13 entries. These are:
;
;		STMODE - Set the 765 mode for I/O
;		STSIZE - Set size of drive
;		STHDRV - Set the head and drive
;		STSECT - Set the sector
;		SPEC   - Do a specify
;		RECAL  - Recalibrate the drive
;		SEEK   - Set the track and seek
;		READ   - Read sector(s)
;		WRITE  - Write sector(s)
;		READID - Read ID of first available sector
;		RETSAD - Return status address location
;		RETDST - Return drive status
;		RETRTC - Return address of Clock/calender
;
;	The following equates must be set or reset for this
;	module on assembly. [=true for yes]
;
;	RTCFLG - CLOCK and DATE update code to be included
;	MOTFLG - MOTOR time out code to be included
;	PERSCI - PERSCI drive code is to be included
;	TANDON - TANDON drive code is to be included
;
;	The following variable equates must be assigned on 
;	assembly.
;
;	DSKDAT - Starting point of Disk control variable
;		 data block. [21 bytes]
;
;	RTCDAT - Starting point of RTC and MOTOR variable
;		 data block. [7 bytes]
;
;	ISTACK - Interrupt stack position
;		 [24 bytes below and 4 bytes ahead]
;
;================================================================
;
;	Equates for INTERRUPT stack 
;	Conditional on equate ISTACK being defined at
;	time of assembly and assumes at least 24 bytes of
;	stack plus 4 bytes ahead of stack.
;
intsp	defl	istack		;interrupt stack pointer
tssp	defl	istack		;tempo storage for sp
tshl	defl	istack+2	;  "      "     "  hl
;
;	Equates for DISK variables 
;	Conditional on equate DSKDAT being defined at
;	time of assembly. Assumes a block of 21 bytes
;	are available.
;
cnport	defl	1ch		;control register port
dmacmd	defl	18h		;dma control port
dmaack	defl	10h		;dma acknowledge port address
tc	defl	14h		;terminal count to 765
fdcst	defl	12		;765 stat port
fdcdt	defl	13		;765 data port
rwdt	defl	dskdat		;765 read/write data ptr
drvsz	defl	dskdat+2	;765 drive size
ddone	defl	dskdat+3	;DMA done flag(not used)
fdone	defl	dskdat+4	;765 done flag
mode	defl	dskdat+5	;765 mode
hdr	defl	dskdat+6	;765 head/drive
trk	defl	dskdat+7	;765 track no (c)
hd	defl	dskdat+8	;765 head (h)
sect	defl	dskdat+9	;765 sector no (r)
n	defl	dskdat+10	;765 bytes/sector (n)
eot	defl	dskdat+11	;765 end of track
gpl	defl	dskdat+12	;765 gap length
dtl	defl	dskdat+13	;765 data length
rsz	defl	dskdat+14	;765 sector size
st0	defl	dskdat+15	;765 st 0
st1	defl	dskdat+16	;765 st 1 (can also be pcn)
st2	defl	dskdat+17	;765 st 2
rc	defl	dskdat+18	;765 result phase c
rh	defl	dskdat+19	;765   "      "   h
rr	defl	dskdat+20	;765   "      "   r
rn	defl	dskdat+21	;765   "      "   n
sp8dat	defl	dskdat+22	;765 specify 8" parameters
sp5dat  defl	dskdat+24	;765 specify 5" parameters
cnbyte  defl	dskdat+26	;control register status byte
	if	motflg
mtm	defl	dskdat+27	;mini-floppy motor time
	endif
;
;	Equates for Motor and RTC variables
;	Conditional on equate RTCDAT being defined of
;	time of assembly.
;
	if	rtcflg
sec	defl	rtcdat+0	;real-time clock, seconds
min	defl	rtcdat+1	;  "	"    "	, minutes
hour	defl	rtcdat+2	;  "	"    "	, hours
year	defl	rtcdat+3	;  "	"    "	, year
day	defl	rtcdat+4	;  "	"    "	, day
month	defl	rtcdat+5	;  "	"    "	, month
	endif
;
;	SET MODE
;
;	STMODE sets the fdc mode for the read/write operations.
;	REG A should have the mode as follows:
;
;		bit 7 set for multi-track
;		 "  6  "   "  mfm operation (double density)
;		 "  5  "   "  skip deleted data address mark
;		bits 4-0 don't care.
;	it uses register pairs bc, de, and hl.
;
stmode:	and	0e0h		;clear bits 4-0
	ld	(mode),a
	ld	hl,(rwdt)
	ld	de,n
	ld	bc,5
	bit	6,a		;mfm set?
	jr	z,stmu		;no
	add	hl,bc		;yes
stmu:	ldir			;set up n,eot,gpl & dtl
	ret
;
;	SET HEAD AND DRIVE
;
;	STHDR sets the head and drive for the disk operations.
;
;		bits 0-2 is drive no
;		bit 3 is head number
;
sthdrv:	ld	(hdr),a
	and	4		;head 1?
	jr	z,shd		;no
	ld	a,1
shd:	ld	(hd),a
	ret
;
;	SET SECTOR 
;
;	STSECT sets the sector for the read/write operations.
;
stsect:	ld	(sect),a
	ret
;
;	RECALIBRATE THE DRIVE
;
;	RECAL is the routine that recalibrates a drive (moves
;	the head to track 0).  the zero flag is set and a=0
;	if no errors occurred, otherwise the zero flag is
;	reset and a is non-zero.
;
recal:
	call	motor
;
	sub	a
	ld	(trk),a		;set track to zero
	ld	a,7
	call	fddrw
	ld	a,(hdr)		;get drive
;
	if	persci
	and	0feh		;reset bit 0
	endif
;
	call	fddrw
	jr	fdcdn
;
;	SEEK
;
;	SEEK sets the track for disk operations and also 
;	seeks the track.  the zero flag is set and a=0 if
;	no errors occurred, otherwise the zero flag is
;	reset and a is non-zero.
;
seek:	ld	(trk),a
;
	call	motor
;
	ld	a,15
	call	fddrw
	ld	a,(hdr)		;get drive
;
	if	persci
	and	0feh		;reset bit 0
	endif
;
	call	fddrw
	ld	a,(trk)
	call	fddrw
	jr	fdcdn
;
;	READ
;
;	READ will read from the floppy disk.  it assumes 
;	that the mode, head/drive, track, and sector have
;	already been set. Enter with hl= address to read
;	data into and  e= the number of sectors to read.
;	It uses all the registers. The zero flag is set
;	and a=0 if no errors occurred, otherwise the zero
;	flag is reset and a is non-zero.
;
read:	ld	a,6
	jr	rw
;
;	WRITE
;
;	WRITE will write to the floppy disk. it assumes that
;	the mode, head/drive, track, and sector have already
;	been set. Enter with hl= address to write data from
;	and e= the number of sectors to write.  It uses all 
;	the registers. The zero flag is set and a=0 if no 
;	errors occurred, otherwise the zero flag is reset 
;	and a is non-zero.
;
write:	ld	a,5
rw:	call	stdma
	call	stfdc		;set up fdc chip
;
;	FDCDN checks the error code after a fdc operation.
;	If a=0 no errors occurred.  This routine is executed 
;	after every fdc interrupt unless the interrupt cause
;	was a change of state in the ready line.  See the
;	fdcint routine for more details.
;
fdcdn:	ld	a,(fdone)
	or	a
	jr	z,fdcdn		;jp if not done
	ld	a,(st0)
	and	0c0h		;normal termination?
	ret
;
;	STDMA sets up the DMA for a read or write operation
;	no. of sectors in reg E is used to multiply the
;	sector size to obtain the full length of bytes to
;	transfer.
;
stdma:	push	af		;save for STFDC
	ld	c,dmacmd	;load control port no of dma
	cp	6		;is it a read or write?
	ld	a,0a3h		;disable the dma
	out	(c),a
	ld	a,0
	ld	(ddone),a	;flag not done
	jr	z,stdma0	;a read
	ld	a,1		;a write, temp decl b as source
 	out	(c),a
	ld	a,0cfh		;load
	out	(c),a
	ld	a,7dh		;Trans A>B
	jr	stdma1
stdma0: ld	a,79h		;trans B>A
stdma1:	out	(c),a		;output command 
	out	(c),l		;output low byte of address
	out	(c),h		;output hi  byte of address
	ld	d,e		;prepare for length output
	ld	e,0
	ld	a,(rsz)		;get size of sector
;
;		fast way to multiply
;		n sectors times sector size
;	
	or	a		;determine if 128 or 256
	jr	z,stdma2	;256
	sra	d
	rr	e		;condition for hi byte
;
stdma2:	dec	de
	out	(c),e		;output low byte of length
	out	(c),d		;output hi  byte of length
	ld	a,0cfh		;load
	out	(c),a
	ld	a,0e0h		;enable
	out	(c),a
	pop	af
	ret
;
;	STFDC sets up the upd765 chip for a read or write
;	operation.  Enter with a equal 5 for write or equal
;	6 for read.
;
stfdc:
;
	push	af
	call	motor
	pop	af
;
	ld	hl,mode
	or	(hl)
	call	fddrw
	ld	b,8
stf1:	inc	hl		;output rest of data
	ld	a,(hl)
	call	fddrw
	djnz	stf1
	ret
;
;	READ ID
;
;	READID reads the first correct id information on 
;	a track. The zero flag is set and a=0 if no errors 
;	occurred, otherwise the zero flag is reset and a 
;	is non-zero.
;
readid:
	call	motor
;
	ld	a,(mode)
	and	40h		;get mf bit from mode
	or	0ah		;and or in command
	call	fddrw
	ld	a,(hdr)
	call	fddrw
	jr	fdcdn
;
;	SET DRIVE SIZE
;
;	STSIZE sets the drive size, either 5.25" or 8", along
;	with the 765 timing constants. It sets up the data
;	pointer and changes the clock speed. It uses the
;	register pairs hl, de, bc and af.
;
stsize:	ld	hl,fm8
	or	a		;check size
	jr	nz,sts0		;jp if 8"
	ld	hl,fm5		;point to 5.25" data
sts0:	ld	(rwdt),hl	;set r/w data pointer
	ld	(drvsz),a	;and size
	ld	a,(cnbyte)
	jr	z,sts1
	res	3,a		;reset bit 3 for 8"
	jr	sts2
sts1:	set	3,a		;set bit 3 for 5.25"
sts2:	out	(cnport),a
	ld	(cnbyte),a
;
;	If size is 8"
;
	ld	de,(sp8dat)
	jp	nz,spec		;do specify
;
;	Size is 5.25"
;
	ld	de,(sp5dat)
	jp	spec		;do specify
;
;	SPECIFY
;
;	SPEC does a specify command.  it expects srt/hut in 
;	d and hut in e(bits 7-1).  dma mode is always chosen.
;
spec:	ld	a,3		;do a fdc specify
	call	fddrw
	ld	a,d
	call	fddrw
	ld	a,e
	and	0feh		;set dma mode
;
;	FDDRW outputs a to the fdc data reg.
;
fddrw:	push	af
fddr:	in	a,(fdcst)
	rlca			;wait for rqm
	jr	nc,fddr
	pop	af
	out	(fdcdt),a
	ret
;
;	MOTOR CONTROL
;
;	MOTOR is the routine that turns on the dc motor
;	on mini-floppy drives.  /MOT is the motor control
;	line, located on THE CONTROL REGISTER, bit 4
;
motor:
	in	a,(tc)		;do a terminal count
	xor	a
	ld	(fdone),a	;reset fdc done flag
;
	if	motflg
	ld	a,(drvsz)
	or	a		;mini drive?
;
	if	persci or tandon
	nop			;ignore drive size
	else
	ret	nz		;return on 8"
	endif
;
	ld	a,30		;set timer
	ld	(mtm),a		;for 30 seconds
	ld	a,(cnbyte)
	bit	4,a		;motor already on?
	ret	z		;yes
	res	4,a		;no,
	ld	(cnbyte),a
	out	(cnport),a	;turn motor on
w2sec:	ld	a,(mtm)		;and wait 2 seconds
	cp	28
	jr	nz,w2sec
	endif
	ret
;	
;
;	read/write set-up data for 5.25" drives.
;
fm5:	db	0		;single density n
	db	18		;  "	  "	eot
	db	5		;  "	  "	gpl
	db	128		;  "	  "	dtl
	db	128		;  "	  "	sector size
	db	1		;double	  "	n
	db	18		;  "	  "	eot
	db	10		;  "	  "	gpl
	db	-1		;  "	  "	dtl
	db	256 mod 256	;  "	  "	sector size
;
;	read/write set-up data for 8" drives.
;
fm8:	db	0		;single density n
	db	26		;  "	  "	eot
	db	7		;  "	  "	gpl
	db	128		;  "	  "	dtl
	db	128		;  "	  "	sector size
	db	1		;double	  "	n
	db	26		;  "	  "	eot
	db	14		;  "	  "	gpl
	db	-1		;  "	  "	dtl
	db	256 mod 256	;  "	  "	sector size
;
;	RETURN STATUS ADDRESS
;
retsad:	ld	hl,st0
	ret
;
;	RETURN DRIVE STATUS
;
;	RETDST returns the status a drive.
;
retdst:
	call	motor
;
	ld	a,4		;sense drive status cmd
	call	fddrw
	ld	a,(hdr)		;get head/drive
	call	fddrw
dst:	in	a,(fdcst)
	rlca			;wait for rqm
	jr	nc,dst
	in	a,(fdcdt)	;get st3
	ret
;
;	RETURN REAL TIME CLOCK PARAMETER ADDRESS
;
retrtc:
	if	rtcflg
	ld	hl,sec
	else
	ld	hl,00h
	endif
	ret
;
;================================================================
;
;	DMA INTERRUPT
;
;	DMAINT is the interrupt routine for the dma chip
;	It assumes that a normal termination has taken place
;	in the transfer.
;
dmaint: ld	(tssp),sp
	ld	sp,intsp	;set up interrupt stack
	push	af
	in	a,(tc)		;do a terminal count
	ld	a,0a3h
	out	(dmacmd),a	;disable dma
	ld	a,-1
	ld	(ddone),a	;signal	dma is done
	pop	af
	ld	sp,(tssp)
	ei
	reti
;
;	765 INTERRUPT
;
;	FDCINT is the interrupt routine for the upd765 chip.
;
fdcint:	ld	(tssp),sp	;save sp
	ex	(sp),hl
	ld	(tshl),hl
	ld	sp,intsp	;set up interrupt stack
	push	af
	push	bc
	ld	hl,st0		;get status
	ld	bc,8*256+fdcdt	;b=08, c=0d
	in	a,(fdcst)
	and	40h		;dio=1?
	jr	nz,fdci2	;yes
	out	(c),b		;do sense interrupt status
fdci1:	in	a,(fdcst)	;get st0,st1,st2,c,h,r & n
	rlca
	jr	nc,fdci1	;wait for rqm
	rlca
	jr	nc,fdci3
fdci2:	ini
	jr	fdci1
fdci3:	djnz	fdci5		;jp if sense int was exec
	call	retdst		;to set seek/rw on 765
fdci4:	ld	hl,fdcdn
	ld	a,-1
	ld	(fdone),a	;set fdc done flag
	jr	fdci6
;
fdci5:	ld	a,(st0)		;sense int was executed
	bit	5,a		;seek end?
	jr	nz,fdci4	;yes
	ld	hl,(tshl)
;
;	Return from fdc interrupt
;
fdci6:
	pop	bc
	pop	af
	ld	sp,(tssp)	;restore stack
	ex	(sp),hl
	ei
	reti
;
;	TIMER
;
;	TIMER is the routine that keeps the real time clock
;	and the MOTOR time out. The real time clock is kept
;	in binary and is a 24 hour clock. TIMER also keeps 
;	a date calender.  It will update the day, month and
;	year whenever the time rolls over the 24 hours.
;	It does not keep track of leap year nor centuries.
;
;	The code is conditional on equate flags, RTCFLG and
;	MOTFLG.
;	
timer:
	if not rtcflg and not motflg
	ei
	reti
;
	else
;
	ld	(tssp),sp	;save sp
	ld	sp,intsp	;set up interrupt stack
	push	af
	push	hl
;
;	MOTOR time out code
;	
	if motflg
	ld	hl,mtm	
	ld	a,(hl)		;get motor time
	or	a		;already zero?
	jr	z,tim0		;yes, don't do anything
	dec	(hl)		;decrement time
	jr	nz,tim0
	ld	a,(cnbyte)	;if zero, turn moter off 
	set	4,a
	ld	(cnbyte),a
	out	(cnport),a
tim0:
	endif
;
;	RTC update code
;
	if rtcflg
	ld	hl,sec
	ld	a,59
	inc	(hl)		;increment seconds
	cp	(hl)
	jr	nc,tim4
	ld	(hl),0
	inc	hl
	inc	(hl)		;increment minutes
	cp	(hl)
	jr	nc,tim4
	ld	(hl),0
	ld	a,23
	inc	hl
	inc	(hl)		;increment hours
	cp	(hl)
	jr	nc,tim4
	ld	(hl),0
	push	bc
	push	de
	inc	hl
	inc	hl		;point to day
	ld	a,(month)	;get month
	ld	c,a
	ld	b,0
	ld	de,dpm-1	;point to day/month table
	ex	de,hl
	add	hl,bc
	ex	de,hl
	ld	a,(de)		;get number of days
	inc	(hl)		;increment day
	cp	(hl)		;next month?
	jr	nc,tim3		;no
	ld	(hl),1		;yes, day=1
	inc	hl
	inc	(hl)		;increment month
	ld	a,12
	cp	(hl)		;next year?
	jr	nc,tim3		;no
	ld	(hl),1		;yes, month=1
	dec	hl
	dec	hl
	inc	(hl)		;year= year+1
tim3:	pop	de
	pop	bc
tim4:
	endif
;
	pop	hl
	pop	af
	ld	sp,(tssp)
	ei
	reti
;
;	DAYS of the MONTH data block for RTC
;
	if rtcflg	
;
dpm:	db	31		;january
	db	28		;febuary
	db	31		;march
	db	30		;april
	db	31		;may
	db	30		;june
	db	31		;july
	db	31		;august
	db	30		;september
	db	31		;october
	db	30		;november
	db	31		;december

	endif			;rtc
	endif			;rtc or motor
;
;	end module
;
