	.z80
	subttl CP/M Disk Parameter Block Module
	page
;
;	Teletek Enterprises, Inc.
;
;================================================================
;		    SYSTEMASTER CP/M 2.2 
;	    DISK PARAMETER BLOCK ALLOCATION MODULE
;
;		   revision number:  B.00
;		  revision date:  05-25-82
;
;	This Module contains the code required to define
;	the CP/M 2.2 disk parameters. This version defines
;	four logical drives that may be eight or five and a
;	quarter inch drives. 128 byte (CP/M 1.4 compatable)
;	single-density, single-sided sectors and 256 byte
;	double-density, single or double-sided sectors are
;	supported. This module is a companion of SMDIOS.MOD.
;
;================================================================
;	cp/m disk variables
;
;	This block of variables resides in the ram area 
;	defined at assembly by equate CPMDAT
;
btrk	defl	cpmdat		;boot track
bsct	defl	cpmdat+1	;boot sector
drsz	defl	cpmdat+2	;current drive size
xlt	defl	cpmdat+3	;sct translation table ptr
ntrks	defl	cpmdat+5	;# of tracks
track	defl	cpmdat+6	;track to seek
hmtrys	defl	cpmdat+7	;home retrys
rwtrys	defl	cpmdat+8	;read/write retrys
rdop	defl	cpmdat+9	;1 for a read operation
;
sekpar	defl	cpmdat+10	;seek parameters
sekdsz	defl	sekpar		;drive size
sekdsk	defl	sekpar+1 	;logical disk number
sektrk	defl	sekpar+2	;track number
npar	defl	4		;# of param
seksec	defl	sekpar+4	;sector number
sekden	defl	sekpar+5	;density
sekcst	defl	sekpar+6	;cp/m sectors/track
skblk	defl	sekpar+7
;
hstpar	defl	skblk		;host parameters
hstdsz	defl	hstpar		;drive size
hstdsk	defl	hstpar+1	;logical disk number
hsttrk	defl	hstpar+2	;track number
hstsec	defl	hstpar+4	;sector number
;
sekhst	defl	hstpar+5	;seek shr secshf
hstact	defl	hstpar+6	;host active flag
hstwrt	defl	hstpar+7	;host written flag
;
unacnt	defl	hstpar+8	;unalloc rec cnt
unadsk	defl	hstpar+9	;last unalloc disk
unatrk	defl	hstpar+10	;last unalloc track
unasec	defl	hstpar+12	;last unalloc sector
;
readop	defl	hstpar+13	;1 if read operation
wrtype	defl	hstpar+14	;write operation type
rsflag	defl	hstpar+15	;read sector flag
erflag	defl	hstpar+16	;error reporting flag
;
dmaadr	defl	hstpar+17	;cp/m dma address
actdma	defl	hstpar+19	;actual dma addr
hsblk	defl	hstpar+21
;
drprtb	defl	hsblk		;drive parameter table
drblk	defl	drprtb+ndrvs*ndpar
;
hstbuf	defl	drblk		;fdc read/write buffer
;
dirbuf	defl	hstbuf+hstsiz	;directory access buffer
;
alv0	defl	dirbuf+128	;allocation vector d.0
alv1	defl	alv0+62		;allocation vector d.1
alv2	defl	alv1+62		;allocation vector d.2
alv3	defl	alv2+62		;allocation vector d.3
csv0	defl	alv3+62		;check vector d.0
csv1	defl	csv0+32		;check vector d.1
csv2	defl	csv1+32		;check vector d.2
csv3	defl	csv2+32		;check vector d.3
;
endcpm	defl	csv3+32-cpmdat	;size of cpmdat block 
;
;================================================================
;
;		drive allocation
;
dpbase:
dpe0:	dw	0,0		;translate table
	dw	0,0		;scratch area
	dw	dirbuf,0	;dir buff,parm block
	dw	csv0,alv0	;check, alloc vectors
;
dpe1:	dw	0,0		;translate table
	dw	0,0		;scratch area
	dw	dirbuf,0	;dir buff,parm block
	dw	csv1,alv1	;check, alloc vectors
;
dpe2:	dw	0,0		;translate table
	dw	0,0		;scratch area
	dw	dirbuf,0	;dir buff,parm block
	dw	csv2,alv2	;check, alloc vectors
;
dpe3:	dw	0,0		;translate table
	dw	0,0		;scratch area
	dw	dirbuf,0	;dir buff,parm block
	dw	csv3,alv3	;check, alloc vectors
;
;
;	sector translation tables
;
xlt8:	db	01,07,13,19,25,05,11,17,23,03,09,15,21
	db	02,08,14,20,26,06,12,18,24,04,10,16,22
;
xlt5:	db	01,07,13,02,08,14,03,09,15
	db	04,10,16,05,11,17,06,12,18
;
;
;	the disk parameter blocks
;	the dpb defines the size of
;	each logical drive to cp/m
;
dpb8d:	;double density, single sided 8" drives
	dw	52		;sec per track
	db	4		;block shift
	db	15		;block mask
	db	1		;extnt mask
	dw	242		;disk size-1
	dw	127		;directory max
	db	192		;alloc0
	db	0		;alloc1
	dw	32		;check size
	dw	2		;offset
;
dpb8dd:	;double density, double sided 8" drives
	dw	52		;sec per track
	db	4		;block shift
	db	15		;block mask
	db	0		;extnt mask
	dw	493		;disk size-1
	dw	127		;directory max
	db	192		;alloc0
	db	0		;alloc1
	dw	32		;check size
	dw	2		;offset
;
dpb8s:	;single density, single sided 8" drives
	dw	26		;sec per track
	db	3		;block shift
	db	7		;block mask
	db	0		;extnt mask
	dw	242		;disk size-1
	dw	63		;directory max
	db	192		;alloc0
	db	0		;alloc1
	dw	16		;check size
	dw	2		;offset
;
;disk parameter blocks for the 5.25" drives
;35 track drives
;
dpb5d:	;double density, single sided
	dw	36		;sec per track
	db	3		;block shift
	db	7		;block mask
	db	0		;extnt mask
	dw	143		;disk size-1
	dw	127		;directory max
	db	240		;alloc0
	db	0		;alloc1
	dw	32		;check size
	dw	3		;offset
;
dpb5dd:	;double density, double sided
	dw	36		;sec per track
	db	4		;block shift
	db	15		;block mask
	db	1		;extnt mask
	dw	149		;disk size-1
	dw	127		;directory max
	db	192		;alloc0
	db	0		;alloc1
	dw	32		;check size
	dw	3		;offset
;
dpb5s:	;single density, single sided
	dw	18		;sec per track
	db	3		;block shift
	db	7		;block mask
	db	0		;extnt mask
	dw	71		;disk size-1
	dw	63		;directory max
	db	192		;alloc0
	db	0		;alloc1
	dw	16		;check size
	dw	3		;offset
;
;	end of module
;
