	.z80
	subttl	SysteMaster Initialization Module
	page
;
;	Teletek Enterprises, Inc.
;
;================================================================
;	       SYSTEMASTER INITIALIZATION MODULE
;
;	This module is the initialization section for the 
;	SYSTEMASTER onboard chips. It initializes 765 and
;	optionally, the CTC, PIO, SIO, and DMA chips depending
;	on equate flags set or reset during assembly.
;
;	ENTRY to this module is INIT. A RET is made after
;	initialization. So, the SP must be initialized prior
;	entering this module.
;
;	The following EQUATES must be set or reset in another
;	module on assembly.
;
;	CTCFLG - CTC initialization [=TRUE for yes]
;	PIOFLG - PIO initialization [=TRUE for yes]
;	SIOFLG - SIO initialization [=TRUE for yes]
;	DMAFLG - DMA initialization [=TRUE for yes]
;	765 interrupt vector is setup regardless. 
;
;	SP8DAT and SP5DAT addresses must be assigned by equates
;	on assembly.
;
;	This module moves the interrupt vectors to RAM
;	location [INTVCT] defined in another module
;	on assembly. [INTVCT] must start at [XXD8H]
;	where XX is the hi byte of address, that is
;	loaded into the Z80.
;
;	Interrupt vector allocation. Must have routines 
;	for each vector group as specified, if initialization
;	flag is true for that group. [JRET] is a general
;	NOP return from interrupt for unused vectors. 
;
;	DMA vector group [D8-DE]
;	   [DC]	- DMAINT - routine to acknowledge end of DMA
;			   BYTE TRANSFER normal completion.
;	   rest are JRET
;
;	SIO vector group [E0-EE]
;	   [E4] - RCAB   - routine to receive input from sio b
;	   [E6] - SRCB	 - routine to ack special rcv cond sio b
;	   [EC] - RCAA   - routine to receive input from sio a
;	   [EE] - SRCA	 - routine to ack special rcv cond sio a
;	   rest are JRET
;
;	CTC vector group [F0-F6]
;	   [F6] - TIMER  - routine to ack second tick of rtc
;	   rest are JRET
;
;	PIO vector group [F8-FC]
;	   [F8] - PIOAIN - routine to recieve input from pio a
;	   [FA] - JRET
;	   [FC] - PIOAOT - routine to ack output to pio a
;
;	765 vector [FE]
;	   [FE] - FDCINT - routine to ack int of 765
; 
;================================================================
;
init:	di			;disable int in case not already
	ld	hl,initbl	;load initialization table addr
in1:	ld	c,(hl)		;load port address
	inc	hl
	ld	a,c		;ck if -1
	inc	a
	jr	z,in2		;if so then done
	ld	b,(hl)
	inc	hl
	otir
	jr	in1		;load in next group
;
in2:	ld	hl,itable	;interrupt table address 
	ld	de,intvct	;ram location address
	ld	bc,itblsz	;size of table
	ldir			;move the table
;
	ld	a,high intvct	;load hi byte of vector table
	ld	i,a
	im	2		;set mode 2
	ld	hl,6f22h
	ld	(sp8dat),hl	;set 8 inch specify word
	ld	hl,0f28h
	ld	(sp5dat),hl	;set 5 inch specify word
	ei
	ret
;
jret:	ei
	reti
;
initbl:
;
;
;	initialize the CTC chip
;
	if	ctcflg
ct	defl	47h		;counter
;
ctcidb:
	db	08h		;ctc-0 control port
	db	3		;no bytes for initialization
	db	0f0h		;interrupt vector
	db	ct		;ch-0 no int's and counter
	db	0		;ch-0 time constant (300 baud)
;
	db	9		;ctc-1 control port
	db	2		;no bytes for initialization
	db	ct		;ch-1 no int's and timer (/16)
	db	4		;ch-1 time constant (19200 baud)
;
	db	10		;ctc-2 control port
	db	2		;no bytes for initialization
	db	27h		;ch-2 no int's and timer (/256)
	db	7dh		;ch-2 time constant (rtc)
;
	db	11		;ctc-1 control port
	db	2		;no bytes for initialization
	db	0c7h		;ch-3 int's and counter
	db	7dh		;ch-3 time constant (second)
	endif
;
;
;	initialize the PIO chip
;
	if	pioflg
pioidb:
;				;setup pio a
	db	05h		;pio a control port
	db	3		;no bytes for initialization
	db	0fch		;interrupt vector
	db	0fh		;set output mode (assume printer)
	db	87h		;enable interrupts
;
	db	04h		;pio data port
	db	1		;number of initialization bytes
	db	80h		;preset Centronix strobe line
;
;				;setup pio b
	db	07h		;pio b control port
	db	4		;no bytes for initialization
	db	0fah		;interrupt vector
	db	0ffh		;set mode 3
	db	0fh		;control word for mode three
	db	07h		;no interrupts
	endif
;
;	initialize the SIO chip
;
	if	sioflg
sioidb:
	db	01h		;sio a control port
	db	9		;no bytes for initialization
	db	18h		;reset channel
	db	14h		;reset ext status
	db	4ch		;x16, 2 stops, no parity
	db	3		;reg 3
	db	0e1h		;8 bits, auto-enable, Rx enable
	db	5		;reg 5
	db	0eah		;8 bits, DTR, RTS, Tx enable
	db	11h		;reset ext status
	db	00h		;all interrupts disabled
;
	db	03h		;sio b control port
	db	11		;no bytes for initialization
	db	18h		;reset channel
	db	2		;reg 2
	db	0e0h		;interrupt vector
	db	14h		;reset ext status
	db	4ch		;x16, 2 stops, no parity
	db	3		;reg 3
	db	0e1h		;8 bits, auto-enable, Rx enable
	db	5		;reg 5
	db	0eah		;8 bits, DTR, RTS, Tx enable
	db	11h		;reset ext status
	db	1ch		;Int Rx, status affects vector
	endif
;
;
;	initialize the DMA chip
;
	if	dmaflg
dmaidb:
	db	18h		;dma control reg port
	db	eodmai-sodmai	;no bytes for initialization
sodmai	equ	$
;		write register 6, [A3]
	db	0a3h		;reset and disable int
;
;		write register 1, [14]
	db	00010100b	;port a is assigned as inc memory
;
;		write register 2, [28]
	db	00101000b	;port b is assigned as fixed port
;
;		write register 4, [95,10,32,D8]
	db	10010101b	;set DMA to BYTE mode
	db	10h		;port address
	db	00110010b	;int control byte
	db	0d8h		;int vector, status affects vctr
;
;		write register 5, [8A]
	db	10001010b	;active high, no wait or auto restart
;
eodmai	equ	$
	endif
;
	db	-1		;end of initialization
;
;	interrupt table (this is moved to ram)
;	the table size is static regardless of initialization
;	options
itable:
;	dma vectors
;
	if 	dmaflg
	dw	jret		;dma int on ready
	dw	jret		;dma int on match
	dw	dmaint		;dma int on end of block
	dw	jret		;dma int on match & eob
	else
	dw	jret,jret,jret,jret
	endif
;
;	sio vectors
;
	if	sioflg
	dw	jret		;sio b, tx buffer empty
	dw	jret		;sio b, ext/statu change
	dw	rcab		;sio b, receive char available
	dw	srcb		;sio b, special receive cond
;
	dw	jret		;sio a, tx buffer empty
	dw	jret		;sio a, ext/statu change
	dw	rcaa		;sio a, receive char available
	dw	srca		;sio a, special receive cond
;
	else
	dw	jret,jret,jret,jret
	dw	jret,jret,jret,jret
	endif
;
;	ctc vectors
;
	if	ctcflg
	dw	jret		;ctc 0
	dw	jret		;ctc 1
	dw	jret		;ctc 2, real time clock tick
	dw	timer		;ctc 3, real time clock second
	else
	dw	jret,jret,jret,jret
	endif
;
;	pio vectors
;
	if	pioflg
	dw	pioain		;pio a input
	dw	jret		;pio b change of status
	dw	pioaot		;pio a output
	else
	dw	jret,jret,jret
	endif
;
;	765 vector
;
	dw	fdcint
;
itblsz	equ	$-itable
;
;	end module
;
