	.z80
	subttl Peripheral I/O Module
	page
;
;	Teletek Enterprises, Inc.
;
;================================================================
;		SYSTEMASTER PERIPHERAL I/O MODULE
;
;		     revision number:  B.00
; 		    revision date:  06-02-82
;
;
;	This module contains the non-disk I/O routines for
;	CP/M 2.2.   SMINIT.MOD contains the the initialization
;	for the SIO, CTC  and PIO chips that are accessed by
;	the routines of this module.
;
;	Entries into this module are:
;
;		CONST	- Console status
;		CONIN	- Console input
;		CONOUT	- Console output
;		LIST	- List output
;		LISTST	- List status
;		READER	- Reader input
;		PUNCH	- Punch output (disabled)
;		PSTR	- Output String (for bios use only)
;		OUTCHR  - Output a character (for bios use only)
; 
;
;================================================================
;
;
;the following parameters are set up by the cold start loader:
;
;
;location in page 0:
;
iobyte	defl	3		;i/o byte (for list only)
;
cnfg	defl	4		;configuration flag*
;
;*the configuration flag is defined as follows
;	(protocol only for serial list device):
;	bit 7 - 0= no protocol, 1= enable protocol
;	bit 6 - if b7= 1, 0= x-on/x-off, 1= etx/ack
;	bit 5 - mask off output byte 0=no 1=yes
;	bit 4 - sio b baud assignment mode
;		0= dynamic,  1= static
;	bits 3-0 - reserved for future use
;
;	port addressing
;
sad	defl	0		;sio a data
sac	defl	1		;sio a control
sas	defl	1		;sio a status
sbd	defl	2		;sio b data
sbc	defl	3		;sio b control
sbs	defl	3		;sio b status
pad	defl	4		;pio a data
pac	defl	5		;pio a control
pbd	defl	6		;pio b data
ctc0	defl	8		;ctc a control
ctc1	defl	9		;ctc b control
;
;ascii characters used:
;
etx	defl	3		;end of text
ack	defl	6		;acknowledge
bell	defl	7		;bell
lf	defl	10		;line feed
ff	defl	12		;form feed (clear screen)
cr	defl	13		;carriage return
xon	defl	11h		;x-on
xoff	defl	13h		;x-off
ctz	defl	1ah		;^z (clear screen)
;
;================================================================
;		console input buffer
;
ptrs:	;pointers for the 64 byte buffer
qout:	db	0
qinp:	db	0
;
queue:	;64 characater type-ahead buffer
	ds	64
quend	equ	$

paor:	db	-1
;
	dw	'si','si','si'
ptssp:	dw	$
;
;	baud rate data tables
;
;	BCTR -  Address of BAUD rate initialization counter byte.
;	        The contents of BCTR must be preset to [2].
;		It is set to [0] when right baud rate is found.
;
;	BPTR -  Address of pointer to BAUD rate CTC parameters
;		in BAUD rate table. The pointer must be 
;		initialized to bottom of the table. Each time
;		the SIO interrupt input routine is accessed, the
;		input byte is checked if it is a CR. The routine
;		will increment up the table until it finds the
;		right baud rate. This implies that the input 
;		stream contains CRs. The table should be setup
;		parameters for baud rate 19200,9600,4800,etc.
;
ct	defl	47h
tm	defl	07
bctr:	db	2	;allows two times per baud.
bend:	db	10	;no of baud rates available + 1
bptr:	dw	baudtb	;baud rate pointer
baudtb:	db	ct,4	;19200 baud
	db	ct,8	; 9600 baud
	db	ct,16	; 4800 baud
	db	ct,32	; 2400 baud
	db	ct,64	; 1200 baud
	db	ct,128	;  600 baud
	db	ct,0	;  300 baud
	db	tm,104	;  150 baud
	db	tm,142	;  110 baud
;
;================================================================
;		start of code
;
const:	;console input status
	ld	hl,(ptrs)	;get inp and out ptrs
	ld	a,l		;see if same
	sub	h
;
retst:	ret	z
	ld	a,-1
	ret
;
conin:	;console input
	call	const
	jr	z,conin
	di
	ld	a,l		;get out ptr
	inc	a		;add one and mask
	and	3fh		;for 64 byte buffer
	ld	(qout),a	;save updated out ptr
	ld	hl,queue	;get buffer start
	add	a,l		;point to byte
	ld	l,a
	jr	nc,conin0
	inc	h
conin0:	ld	a,(hl)		;get the byte
	ei
	and	07fh
	ret
;
conout:	;console output
	in	a,(sbc)
	and	4
	jr	z,conout
	ld	a,c
	out	(sbd),a
	ret
;
list:	;listing device output
	ld	a,(iobyte)
	rlca			;lpt: or ul1:?
	jr	nc,conout	;no, tty: or crt:
	rlca			;parallel (ul1:)?
	jr	c,plist		;yes
slist:	;list on the serial device (lpt:)
	jr	outser
jrx:	jr	xonxof		;jump to protocol routine
;
;do the ext/ack protocol
	ld	hl,ncout
	inc	(hl)		;128 chars out?
	jp	p,outser	;no
	ld	(hl),0		;yes
	push	bc
	ld	c,etx		;send etx
	call	outser
	pop	bc
exac:	call	reader
	cp	ack		;wait for ack
	jr	nz,exac
;
outser:	;output to serial list
	call	listst
	jr	z,outser
	ld	a,c
andser:	and	7fh			;[AND 7FH for some printers]
	out	(sad),a
	ret
;
ncout:	db	0		;# char's output
;
;do the x-on/x-off protocol
xonxof:	call	rdst		;any input char?
	jr	z,outser	;no
	call	reader		;yes, get it
	cp	xoff		;x-off?
	jr	nz,outser	;no, ignore
wtxon:	call	reader		;yes, now wait for x-on
	cp	xon
	jr	z,outser
	jr	wtxon
;
;
plist:	;list on the parallel device
	call	paost
	jr	z,plist
	xor	a
	ld	(paor),a	;reset status pio int flag
	ld	a,c
	and	7fh		;strobe bit 7
	out	(pad),a
	or	80h		;turn off strobe
	out	(pad),a
	ret
;
;
listst:	;listing device status
	ld	a,(iobyte)
	rlca			;serial or parallel?
	jp	nc,const	;no, must be console
	rlca			;serial?
	jr	nc,lsts		;yes
	;jr	paost		;no, parallel
;
paost:	;pio a output status
	ld	a,(paor)
	or	a
	ret
;
lsts:	;serial list status
	in	a,(sas)
	and	4
	jp	retst
;
;
reader:	;reader, input from sio a
	call	rdst
	jr	z,reader
	in	a,(sad)
	and	07fh
	ret
;
rdst:	;reader input status
	in	a,(sac)
	and	1
	jp	retst
;
punch: ;punch, (disabled) sends bytes to pio b w/ no hand sh.
enpun:	ret
	ld	a,c
	out	(pbd),a
	ret
;
pstr:	;print an ascii message terminated by bit 7= 1
	ex	(sp),hl		;point to message
pms0:	ld	a,(hl)
	inc	hl
	call	outchr
	or	a		;end of message?
	jp	p,pms0		;no
	ex	(sp),hl
	ret
;
outchr:	ld	c,a
	jp	conout
;
;================================================================
;
;	Interrupt service routines for SIO B
;
;	SIO B will return nothing until the baud rate is
;	ok.  On initialization BCTR contains 2 and will
;	remain 2 or 1 until there is a match with a CR.
;	When there is match BCTR will be permenantly changed
;	to 0.
;
rcab:	ld	(ptssp),sp
	ld	sp,ptssp
	push	af
	push	bc
	push	hl
	ld	a,(bctr)
	or	a
	in	a,(sbd)
	ld	c,a		;save the byte out of the way.
	jr	z,rcab00	;ck if console baud rate matched
	and 	7fh	
	cp	cr		;see if CR
	jr	nz,rcab01	;no match
	xor	a		; a match
	ld	(bctr),a	;indicate it and ret
	jr	rcab00		;load cr into buffer
;
rcab01:
	ld	hl,bctr		;dec baud counter
	dec	(hl)
	jr	nz,rcab3
	ld	(hl),2		;reset counter
	ld	a,(bend)	;determine if end
	dec	a
	ld	(bend),a
	jr	nz,rcab02
	ld	a,9		;start over
	ld	(bend),a
	ld	hl,baudtb
	jr	rcab03
;
rcab02: ld	hl,(bptr)	;change baud rate
rcab03:	ld	bc,2*256+ctc1
	otir
	ld	(bptr),hl
	jr	rcab3
;
rcab00:	ld	hl,(ptrs)	;load inp and out buffer ptrs.
	ld	a,h		;get inp ptr
	cp	63		;see if top of buffer
	ld	a,l		;get out ptr in any case
	jr	nz,rcab0	;no
	or	a		;see if out ptr is bottom of buffer
	jr	nz,rcab1	;no, load byte
	jr	rcab3		;yes, ignore byte
;
rcab0:	sub	h		;ck if out ptr one byte
	dec	a		;ahead of inp ptr
	jr	z,rcab3		;yes, ignore byte.
;
rcab1:	ld	a,h		;get inp ptr
	inc	a		;add one and mask for 64 byte buffer
	and	3fh
	ld	(qinp),a	;put away updated inp ptr
	ld	hl,queue	;get buffer start
	add	a,l		;offset to position
	ld	l,a
	jr	nc,rcab2
	inc	h
rcab2:	ld	(hl),c		;put away byte
;
rcab3:	pop	hl
	pop	bc
retisr:	pop	af
	ld	sp,(ptssp)
retena:	ei
	reti
;
srcb:	;special receive condition
	ld	(ptssp),sp
	ld	sp,ptssp
	push	af
	ld	a,30h
	out	(sbc),a
	jr	retisr
;
;===============================================
; sio a interrupt routines
;
rcaa:	jr	retena
;
srca:	;special receive condition
	ld	(ptssp),sp
	ld	sp,ptssp
	push	af
	ld	a,30h
	out	(sac),a
	jr	retisr
;
;===========================================
; pio a interrupt routines
;
pioain:	;pio a input
	jr	retena
;
pioaot:	;pio a output
	ld	(ptssp),sp
	ld	sp,ptssp
	push	af
	ld	a,-1
	ld	(paor),a
	jr	retisr
;
;
;	end of module
;
